@@ -28,9 +28,9 @@
 
 using namespace clover;
 
-kernel::kernel(program &prog, const std::string &name,
+kernel::kernel(clover::program &prog, const std::string &name,
                const std::vector<module::argument> &margs) :
-   prog(prog), _name(name), exec(*this) {
+   program(prog), _name(name), exec(*this) {
    for (auto &marg : margs) {
       if (marg.type == module::argument::scalar)
          _args.emplace_back(new scalar_argument(marg.size));
@@ -57,7 +57,7 @@ template<typename V>
 static inline std::vector<uint>
 pad_vector(command_queue &q, const V &v, uint x) {
    std::vector<uint> w { v.begin(), v.end() };
-   w.resize(q.dev.max_block_size().size(), x);
+   w.resize(q.device().max_block_size().size(), x);
    return w;
 }
 
@@ -66,7 +66,7 @@ kernel::launch(command_queue &q,
                const std::vector<size_t> &grid_offset,
                const std::vector<size_t> &grid_size,
                const std::vector<size_t> &block_size) {
-   const auto m = prog.binary(q.dev);
+   const auto m = program().binary(q.device());
    const auto reduced_grid_size =
       map(divides(), grid_size, block_size);
    void *st = exec.bind(&q);
@@ -130,7 +130,7 @@ std::vector<size_t>
 kernel::optimal_block_size(const command_queue &q,
                            const std::vector<size_t> &grid_size) const {
    return factor::find_grid_optimal_factor<size_t>(
-      q.dev.max_threads_per_block(), q.dev.max_block_size(),
+      q.device().max_threads_per_block(), q.device().max_block_size(),
       grid_size);
 }
 
@@ -151,7 +151,7 @@ kernel::args() const {
 
 const module &
 kernel::module(const command_queue &q) const {
-   return prog.binary(q.dev);
+   return program().binary(q.device());
 }
 
 kernel::exec_context::exec_context(kernel &kern) :
@@ -164,11 +164,11 @@ kernel::exec_context::~exec_context() {
 }
 
 void *
-kernel::exec_context::bind(command_queue *_q) {
+kernel::exec_context::bind(intrusive_ptr<command_queue> _q) {
    std::swap(q, _q);
 
    // Bind kernel arguments.
-   auto &m = kern.prog.binary(q->dev);
+   auto &m = kern.program().binary(q->device());
    auto margs = find(name_equals(kern.name()), m.syms).args;
    auto msec = find(type_equals(module::section::text), m.secs);
 
@@ -313,7 +313,7 @@ kernel::scalar_argument::bind(exec_context &ctx,
    auto w = v;
 
    extend(w, marg.ext_type, marg.target_size);
-   byteswap(w, ctx.q->dev.endianness());
+   byteswap(w, ctx.q->device().endianness());
    align(ctx.input, marg.target_align);
    insert(ctx.input, w);
 }
@@ -327,7 +327,7 @@ kernel::global_argument::set(size_t size, const void *value) {
    if (size != sizeof(cl_mem))
       throw error(CL_INVALID_ARG_SIZE);
 
-   buf = &obj<buffer>(*(cl_mem *)value);
+   buf = pobj<buffer>(value ? *(cl_mem *)value : NULL);
    _set = true;
 }
 
@@ -335,8 +335,23 @@ void
 kernel::global_argument::bind(exec_context &ctx,
                               const module::argument &marg) {
    align(ctx.input, marg.target_align);
-   ctx.g_handles.push_back(allocate(ctx.input, marg.target_size));
-   ctx.g_buffers.push_back(buf->resource(*ctx.q).pipe);
+
+   if (buf) {
+      const resource &r = buf->resource(*ctx.q);
+      ctx.g_handles.push_back(ctx.input.size());
+      ctx.g_buffers.push_back(r.pipe);
+
+      // How to handle multi-demensional offsets?
+      // We don't need to.  Buffer offsets are always
+      // one-dimensional.
+      auto v = bytes(r.offset[0]);
+      extend(v, marg.ext_type, marg.target_size);
+      byteswap(v, ctx.q->device().endianness());
+      insert(ctx.input, v);
+   } else {
+      // Null pointer.
+      allocate(ctx.input, marg.target_size);
+   }
 }
 
 void
@@ -363,7 +378,7 @@ kernel::local_argument::bind(exec_context &ctx,
    auto v = bytes(ctx.mem_local);
 
    extend(v, module::argument::zero_ext, marg.target_size);
-   byteswap(v, ctx.q->dev.endianness());
+   byteswap(v, ctx.q->device().endianness());
    align(ctx.input, marg.target_align);
    insert(ctx.input, v);
 
@@ -379,27 +394,35 @@ kernel::constant_argument::set(size_t size, const void *value) {
    if (size != sizeof(cl_mem))
       throw error(CL_INVALID_ARG_SIZE);
 
-   buf = &obj<buffer>(*(cl_mem *)value);
+   buf = pobj<buffer>(value ? *(cl_mem *)value : NULL);
    _set = true;
 }
 
 void
 kernel::constant_argument::bind(exec_context &ctx,
                                 const module::argument &marg) {
-   auto v = bytes(ctx.resources.size() << 24);
-
-   extend(v, module::argument::zero_ext, marg.target_size);
-   byteswap(v, ctx.q->dev.endianness());
    align(ctx.input, marg.target_align);
-   insert(ctx.input, v);
 
-   st = buf->resource(*ctx.q).bind_surface(*ctx.q, false);
-   ctx.resources.push_back(st);
+   if (buf) {
+      resource &r = buf->resource(*ctx.q);
+      auto v = bytes(ctx.resources.size() << 24 | r.offset[0]);
+
+      extend(v, module::argument::zero_ext, marg.target_size);
+      byteswap(v, ctx.q->device().endianness());
+      insert(ctx.input, v);
+
+      st = r.bind_surface(*ctx.q, false);
+      ctx.resources.push_back(st);
+   } else {
+      // Null pointer.
+      allocate(ctx.input, marg.target_size);
+   }
 }
 
 void
 kernel::constant_argument::unbind(exec_context &ctx) {
-   buf->resource(*ctx.q).unbind_surface(*ctx.q, st);
+   if (buf)
+      buf->resource(*ctx.q).unbind_surface(*ctx.q, st);
 }
 
 void
@@ -417,7 +440,7 @@ kernel::image_rd_argument::bind(exec_context &ctx,
    auto v = bytes(ctx.sviews.size());
 
    extend(v, module::argument::zero_ext, marg.target_size);
-   byteswap(v, ctx.q->dev.endianness());
+   byteswap(v, ctx.q->device().endianness());
    align(ctx.input, marg.target_align);
    insert(ctx.input, v);
 
@@ -445,7 +468,7 @@ kernel::image_wr_argument::bind(exec_context &ctx,
    auto v = bytes(ctx.resources.size());
 
    extend(v, module::argument::zero_ext, marg.target_size);
-   byteswap(v, ctx.q->dev.endianness());
+   byteswap(v, ctx.q->device().endianness());
    align(ctx.input, marg.target_align);
    insert(ctx.input, v);
 
